<?php
class Connection {
  private $db_hostname;
  private $db_username;
  private $db_password;
  private $db_dbname;
  private $query_cache_store;
  private $cache_timeout;
  private $last_query_cached_time;
  public $allow_stale = false;
  public $db_conn;
  public $force_refresh = false;
  public $debug = true;
  
  function __construct($host='localhost', $user='root', $pass='cisco123', $db='cjcs_web')
  {
    global $_GET;
    $this->db_conn = NULL;
    $this->db_hostname = $host;
    $this->db_username = $user;
    $this->db_password = $pass;
    $this->db_dbname = $db;
    if (array_key_exists('debug',$_GET))
    {
      $this->debug = true;
    }
    if (array_key_exists('force_refresh',$_GET))
    {
      $this->force_refresh = true;
    }
    if (!$this->db_connect())
    {
      error_log("db_connect() failed!");
    }
    $this->query_cache_store = $this->get("QUERY_CACHE_DIR",getcwd()."/../querycache");
    $this->cache_timeout = (int)$this->get("QUERY_CACHE_DURATION",86400);
    if (!file_exists($this->query_cache_store))
    {
      if (!mkdir($this->query_cache_store, 0700, true))
      {
        error_log("Unable to create directory:[".$this->query_cache_store."]");
      }
    }
  }

  function __destruct() 
  {
    $this->db_close();
    exit();
  }

  private function db_connect() 
  {
    if (!$this->db_close())
    {
      return false;
    }
    $this->db_conn = mysql_connect($this->db_hostname,$this->db_username,$this->db_password);
    if (empty($this->db_conn)) 
    {
      error_log("Cannot connect to server:[".$this->db_hostname."] error:[".mysql_error()."]");
      return false;
    }
    if (!$this->set_db($this->db_dbname))
    {
      return false;
    }
    return true;
  }
  private function db_close() 
  {
    if (!empty($this->db_conn)) 
    {
      if (!mysql_close($this->db_conn))
      {
        error_log("db_close() failed!");
        return false;
      }
      $this->db_conn = NULL;
    }
    return true;
  }
  public function set_db($db) 
  {
    if (!mysql_select_db($db,$this->db_conn)) 
    {
      error_log("Cannot select database:[".$db."] error:[".mysql_error()."]");
      return false;
    }
    $this->db_dbname = $db;
    return true;
  }
  public function get($key,$default=null)
  {
    if (empty($this->db_conn))
    {
      $this->db_connect();
    }
    $r = $this->query("SELECT `value` FROM `configuration` WHERE `key_name` = '".$key."' LIMIT 1;",false);
    return (sizeof($r) == 0) ? $default : $r[0]['value'];
  }
  public function query($query,$perform_cache=true) 
  {
    if (empty($query))
    {
      return $ret;
    }
    if ($this->debug)
    {
      error_log("[QUERY]:[".$query."]");
    }
    if (preg_match('/\b(UPDATE|INSERT|DELETE|DROP|CREATE)\b/i',$query) == 1)
    {
      // query is not a SELECT so we should pass it directly to SQL
      return $this->query_sql($query,false);
    }
    else if ($this->isCached($query) && !$this->force_refresh)
    {
      // query could be cached
      return $this->query_cache($query,$perform_cache);
    }
    else
    {
      // query not cached, hit SQL
      return $this->query_sql($query,$perform_cache);
    }
  }
  function query_cache($query,$perform_cache=true)
  {
    $diff = time() - $this->getCachedTime($query);
    if ($this->debug)
    {
      error_log("[CACHE HIT]");
      error_log("[CACHE AGE]:[".$diff."]");
    }
    if ($this->allow_stale)
    {
      if ($this->debug)
      {
        error_log("[ALLOW STALE]");
      }
      return $this->getCachedData($query);
    }
    else if ($diff < $this->cache_timeout)
    {
      if ($this->debug)
      {
        error_log("[NOT STALE]");
      }
      return $this->getCachedData($query);
    }
    else
    {
      if ($this->debug)
      {
        error_log("[STALE FORCE REFRESH]");
      }
      return $this->query_sql($query,$perform_cache);
    }  
  }
  function purge_query_cache()
  {
    $dir = scandir($this->query_cache_store);
    for ($i =0;$i<sizeof($dir);$i++)
    {
      $file = $this->query_cache_store."/".$dir[$i];
      if (file_exists($file) && is_file($file) && (filemtime($file) < time()-$this->cache_timeout))
      {
        if ($this->debug)
        {
          error_log("Deleting file:[".$file."]");
        }
        unlink($file);
      }
    }  
  }
  function query_sql($query,$perform_cache=true)
  {
    $ret = array();
    if ($this->debug)
    {
      error_log("[SQL HIT]");
    }
    $r = mysql_query($query,$this->db_conn);
    if (!$r)
    {
      error_log(mysql_error());
      error_log("Cannot Execute Query: ".$query);
      return false;
    }
    if (is_bool($r))
    {
      // when calling SQL with INSERT, UPDATE, DELETE, DROP, CREATE mysql_query returns TRUE
      // there are no query results to enumerate into the array, so simply return TRUE
      return $r;
    }
    for ($i=0; $i<mysql_num_rows($r); $i++ ) 
    {
      $ret[$i] = mysql_fetch_array($r);
      //echo $ret[your_row_id]['your_field_name']; 
    }
    if ($perform_cache)
    {
      $this->cacheData($query,$ret);
    }
    return $ret;
  }
  function getCachedFile($query)
  {     
    $md = md5($query);
    $file = $this->query_cache_store."/".$md."."."sqlcache";
    return $file;
  }   
  function isCached($query)
  {  
    $file = $this->getCachedFile($query);
    return (file_exists($file));
  }
  function getCachedData($query)
  {
    $file = $this->getCachedFile($query);
    if (file_exists($file))
    {
      $data = unserialize(implode('',file($file)));
      $this->last_query_cache_time = $this->getCachedTime($query);
      return $data;
    }
    return null;
  }
  function getCachedTime($query)
  {
    $file = $this->getCachedFile($query);
    if (file_exists($file))
    {
      return filemtime($file);
    }
    return null;
  }
  function cacheData($query,$data)
  {
    $file = $this->getCachedFile($query);
    $output = serialize($data);
    $fp = fopen($file,"w");
    if (flock($fp, LOCK_EX))
    {
      fputs($fp, $output);
      flock($fp, LOCK_UN);
    }
    fclose($fp);
  }
  // Session functions here
  function sess_open($sess_path, $sess_name)
  {
    if (empty($this->db_conn))
    {
      $this->db_connect();
    }
    return true;
  }
  function sess_close()
  {
    return true;
  }
  function sess_read($sess_id)
  {
    if (empty($this->db_conn))
    {
      $this->db_connect();
    }
    $r = $this->query("SELECT data FROM sessions WHERE session_key = '$sess_id' ORDER BY modified_at DESC LIMIT 1;");
    $time = new DateTime;
    $time = $time->format('Y-m-d H:i:s');
    if (sizeof($r) == 0)
    {
      if (!$this->query("INSERT INTO sessions (session_key, modified_at) VALUES ('$sess_id', '$time');"))
      {
        error_log("Unable to create session_key!");
      }
      return '';
    }
    else
    {
      if (!$this->query("UPDATE sessions SET modified_at = '$time' WHERE session_key = '$sess_id';"))
      {
        error_log("Unable to update session_key!");
      }
      return $r[0]['data'];
    }
  }
  function sess_write($sess_id, $data)
  {
    if (empty($this->db_conn))
    {
      $this->db_connect();
    }
    $time = new DateTime;
    $time = $time->format('Y-m-d H:i:s');
    $esc = mysql_real_escape_string($data,$this->db_conn);
    if (!$this->query("UPDATE sessions SET data = '$esc', modified_at = '$time' WHERE session_key = '$sess_id';"))
    {
        error_log("Unable to write data to session_key!");
    }
    return true;
  }
  function sess_destroy($sess_id)
  {
    if (empty($this->db_conn))
    {
      $this->db_connect();
    }
    if (!$this->query("DELETE FROM sessions WHERE session_key = '$sess_id';"))
    {
        error_log("Unable to destroy session_key!");
    }
    return true;
  }
  function sess_gc($sess_maxlifetime)
  {
    if (empty($this->db_conn))
    {
      $this->db_connect();
    }
    $time = new DateTime;
    $time = $time->format('Y-m-d H:i:s');
    if (!$this->query("DELETE FROM sessions WHERE 'modified_at' + '$sess_maxlifetime' < '$time';"))
    {
        error_log("Unable to garbage collect session_key!");
    }
    return true;
  }
  function redirect_to($page)
  {
    $host = $_SERVER['HTTP_HOST'];
    $uri = rtrim(dirname($_SERVER['PHP_SELF']), '/\\');
    header("Location: http://$host$uri/$page");
  }
  
  function security_check()
  {
    session_set_save_handler(
      array($this,"sess_open"),
      array($this,"sess_close"),
      array($this,"sess_read"),
      array($this,"sess_write"),
      array($this,"sess_destroy"),
      array($this,"sess_gc")
    );
    session_start();
    if (isset($_SESSION['username']) && isset($_SESSION['password']))
    {
      // already authenticated, get cached credentials
      $in_username = $_SESSION['username'];
      $in_password = $_SESSION['password'];
    }
    if (!empty($_POST))
    {
      if (!empty($_POST['username']))
      {
        $in_username = $_POST['username'];
      }
      if (!empty($_POST['username']))
      {
        // one-way encrypt the password with the salt set to the password sting
        $in_password = crypt($_POST['password'],$_POST['password']);
      }
    }
    if (empty($in_username) || empty($in_password))
    {
      // bad request for authentication, redirect to login page
      $this->redirect_to("login.php?Err=2&Msg=Your Session Timed Out");
    }
    if (!empty($in_username))
    {
      $r = $this->query('SELECT pkid,username,crypt_password FROM users WHERE username = "'.$in_username.'" ORDER BY modified_at DESC LIMIT 1');
      if (sizeof($r) != 1)
      {
        // TODO: no or multiple user(s) by this name; add with read-only rights
        if (!$conn->this("INSERT INTO users SET username = '".$in_username."', crypt_password = '".$db_password."'"))
        {
          error_log("Unable to create username:[".$in_username."]!");
        }
        $r = $conn->this('SELECT pkid,username,crypt_password FROM users WHERE username = "'.$in_username.'" ORDER BY modified_at DESC LIMIT 1');
      }
    
      if ($in_password == $r[0]['crypt_password'])
      {
        // the password is valid, save the session as authenticated and allow flow through
        $_SESSION['username'] = $r[0]['username'];
        $_SESSION['password'] = $r[0]['crypt_password'];
        $_SESSION['pkid'] = $r[0]['pkid'];
        // TODO: perform group level permission lookup for protected areas
       // $_SESSION['group_acl'] = $r[0]['group_acl'];
      }
      else
      {
        error_log("Username:[".$in_username."] supplied an invalid password!");
        $this->redirect_to("login.php?Err=1&Msg=Invalid Username or Password");
      }
    }
  }
  function is_admin()
  {
    // TODO: perfom lookup on $_SESSION['username'] to see which groups they belong?
    return true;
  }
}


$conn = new Connection();
?>
